@@ -17,13 +17,12 @@
 
 package org.apache.jmeter;
 
-import java.awt.event.ActionEvent;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
-import java.io.FileReader;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.Reader;
 import java.net.Authenticator;
 import java.net.DatagramPacket;
 import java.net.DatagramSocket;
@@ -31,12 +30,15 @@ import java.net.InetAddress;
 import java.net.MalformedURLException;
 import java.net.SocketException;
 import java.nio.charset.StandardCharsets;
-import java.text.SimpleDateFormat;
+import java.nio.file.Files;
+import java.time.Instant;
+import java.time.ZoneId;
+import java.time.format.DateTimeFormatter;
+import java.time.format.FormatStyle;
+import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Date;
 import java.util.Enumeration;
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
@@ -52,8 +54,6 @@ import javax.script.ScriptEngine;
 import javax.script.ScriptEngineFactory;
 import javax.script.ScriptEngineManager;
 import javax.script.ScriptException;
-import javax.swing.JTree;
-import javax.swing.tree.TreePath;
 
 import org.apache.commons.cli.avalon.CLArgsParser;
 import org.apache.commons.cli.avalon.CLOption;
@@ -70,17 +70,9 @@ import org.apache.jmeter.engine.RemoteJMeterEngineImpl;
 import org.apache.jmeter.engine.StandardJMeterEngine;
 import org.apache.jmeter.engine.TreeCloner;
 import org.apache.jmeter.exceptions.IllegalUserActionException;
-import org.apache.jmeter.gui.GuiPackage;
-import org.apache.jmeter.gui.MainFrame;
-import org.apache.jmeter.gui.action.ActionNames;
-import org.apache.jmeter.gui.action.ActionRouter;
-import org.apache.jmeter.gui.action.Load;
 import org.apache.jmeter.gui.action.LoadRecentProject;
-import org.apache.jmeter.gui.action.LookAndFeelCommand;
-import org.apache.jmeter.gui.tree.JMeterTreeListener;
 import org.apache.jmeter.gui.tree.JMeterTreeModel;
 import org.apache.jmeter.gui.tree.JMeterTreeNode;
-import org.apache.jmeter.gui.util.FocusRequester;
 import org.apache.jmeter.plugin.JMeterPlugin;
 import org.apache.jmeter.plugin.PluginManager;
 import org.apache.jmeter.report.config.ConfigurationException;
@@ -98,11 +90,10 @@ import org.apache.jmeter.threads.RemoteThreadsListenerTestElement;
 import org.apache.jmeter.util.BeanShellInterpreter;
 import org.apache.jmeter.util.BeanShellServer;
 import org.apache.jmeter.util.JMeterUtils;
+import org.apache.jmeter.util.SecurityProviderLoader;
 import org.apache.jmeter.util.ShutdownClient;
 import org.apache.jorphan.collections.HashTree;
 import org.apache.jorphan.collections.SearchByClass;
-import org.apache.jorphan.gui.ComponentUtil;
-import org.apache.jorphan.gui.JMeterUIDefaults;
 import org.apache.jorphan.reflect.ClassTools;
 import org.apache.jorphan.util.HeapDumper;
 import org.apache.jorphan.util.JMeterException;
@@ -113,8 +104,6 @@ import org.apache.logging.log4j.core.config.Configurator;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.thoughtworks.xstream.converters.ConversionException;
-
 /**
  * Main JMeter class; processes options and starts the GUI, non-GUI or server as appropriate.
  */
@@ -359,79 +348,6 @@ public class JMeter implements JMeterPlugin {
     }
 
 
-    /**
-     * Starts up JMeter in GUI mode
-     */
-    private void startGui(String testFile) {
-        System.out.println("================================================================================");//NOSONAR
-        System.out.println("Don't use GUI mode for load testing !, only for Test creation and Test debugging.");//NOSONAR
-        System.out.println("For load testing, use CLI Mode (was NON GUI):");//NOSONAR
-        System.out.println("   jmeter -n -t [jmx file] -l [results file] -e -o [Path to web report folder]");//NOSONAR
-        System.out.println("& increase Java Heap to meet your test requirements:");//NOSONAR
-        System.out.println("   Modify current env variable HEAP=\"-Xms1g -Xmx1g -XX:MaxMetaspaceSize=256m\" in the jmeter batch file");//NOSONAR
-        System.out.println("Check : https://jmeter.apache.org/usermanual/best-practices.html");//NOSONAR
-        System.out.println("================================================================================");//NOSONAR
-
-        JMeterUIDefaults.INSTANCE.install();
-
-        String jMeterLaf = LookAndFeelCommand.getPreferredLafCommand();
-        try {
-            log.info("Setting LAF to: {}", jMeterLaf);
-            LookAndFeelCommand.activateLookAndFeel(jMeterLaf);
-        } catch (IllegalArgumentException ex) {
-            log.warn("Could not set LAF to: {}", jMeterLaf, ex);
-        }
-        // SplashWindow is created after LaF activation. Otherwise it would cause splash flicker
-        SplashScreen splash = new SplashScreen();
-        splash.showScreen();
-        splash.setProgress(10);
-        JMeterUtils.applyHiDPIOnFonts();
-        PluginManager.install(this, true);
-
-        JMeterTreeModel treeModel = new JMeterTreeModel();
-        splash.setProgress(30);
-        JMeterTreeListener treeLis = new JMeterTreeListener(treeModel);
-        final ActionRouter instance = ActionRouter.getInstance();
-        instance.populateCommandMap();
-        splash.setProgress(60);
-        treeLis.setActionHandler(instance);
-        GuiPackage.initInstance(treeLis, treeModel);
-        splash.setProgress(80);
-        MainFrame main = new MainFrame(treeModel, treeLis);
-        splash.setProgress(100);
-        ComponentUtil.centerComponentInWindow(main, 80);
-        main.setLocationRelativeTo(splash);
-        main.setVisible(true);
-        main.toFront();
-        instance.actionPerformed(new ActionEvent(main, 1, ActionNames.ADD_ALL));
-        if (testFile != null) {
-            try {
-                File f = new File(testFile);
-                log.info("Loading file: {}", f);
-                FileServer.getFileServer().setBaseForScript(f);
-
-                HashTree tree = SaveService.loadTree(f);
-
-                GuiPackage.getInstance().setTestPlanFile(f.getAbsolutePath());
-
-                Load.insertLoadedTree(1, tree);
-            } catch (ConversionException e) {
-                log.error("Failure loading test file", e);
-                JMeterUtils.reportErrorToUser(SaveService.CEtoString(e));
-            } catch (Exception e) {
-                log.error("Failure loading test file", e);
-                JMeterUtils.reportErrorToUser(e.toString());
-            }
-        } else {
-            JTree jTree = GuiPackage.getInstance().getMainFrame().getTree();
-            TreePath path = jTree.getPathForRow(0);
-            jTree.setSelectionPath(path);
-            FocusRequester.requestFocus(jTree);
-        }
-        splash.setProgress(100);
-        splash.close();
-    }
-
     /**
      * Takes the command line arguments and uses them to determine how to
      * startup JMeter.
@@ -462,10 +378,12 @@ public class JMeter implements JMeterPlugin {
         try {
             initializeProperties(parser); // Also initialises JMeter logging
 
+            SecurityProviderLoader.addSecurityProvider(JMeterUtils.getJMeterProperties());
+
             Thread.setDefaultUncaughtExceptionHandler(
                     (Thread t, Throwable e) -> {
                     if (!(e instanceof ThreadDeath)) {
-                        log.error("Uncaught exception in thread " + t, e);
+                        log.error("Uncaught exception in thread {}", t, e);
                         System.err.println("Uncaught Exception " + e + " in thread " + t + ". See log file for details.");//NOSONAR
                     }
             });
@@ -504,11 +422,10 @@ public class JMeter implements JMeterPlugin {
             }
 
             // Set some (hopefully!) useful properties
-            long now=System.currentTimeMillis();
-            JMeterUtils.setProperty("START.MS",Long.toString(now));// $NON-NLS-1$
-            Date today=new Date(now); // so it agrees with above
-            JMeterUtils.setProperty("START.YMD",new SimpleDateFormat("yyyyMMdd").format(today));// $NON-NLS-1$ $NON-NLS-2$
-            JMeterUtils.setProperty("START.HMS",new SimpleDateFormat("HHmmss").format(today));// $NON-NLS-1$ $NON-NLS-2$
+            Instant now = Instant.now();
+            JMeterUtils.setProperty("START.MS",Long.toString(now.toEpochMilli()));// $NON-NLS-1$
+            JMeterUtils.setProperty("START.YMD", getFormatter("yyyyMMdd").format(now));// $NON-NLS-1$ $NON-NLS-2$
+            JMeterUtils.setProperty("START.HMS", getFormatter("HHmmss").format(now));// $NON-NLS-1$ $NON-NLS-2$
 
             if (parser.getArgumentById(VERSION_OPT) != null) {
                 displayAsciiArt();
@@ -544,7 +461,9 @@ public class JMeter implements JMeterPlugin {
                     ReportGenerator generator = new ReportGenerator(reportFile, null);
                     generator.generate();
                 } else if (parser.getArgumentById(NONGUI_OPT) == null) { // not non-GUI => GUI
-                    startGui(testFile);
+                    PluginManager.install(this, true);
+                    String initialTestFile = testFile;
+                    JMeterGuiLauncher.startGui(initialTestFile);
                     startOptionalServers();
                 } else { // NON-GUI must be true
                     extractAndSetReportOutputFolder(parser, deleteResultFile);
@@ -578,6 +497,10 @@ public class JMeter implements JMeterPlugin {
         }
     }
 
+    private static DateTimeFormatter getFormatter(String pattern) {
+        return DateTimeFormatter.ofPattern(pattern).withZone(ZoneId.systemDefault());
+    }
+
     /**
      * Extract option JMeter#REPORT_OUTPUT_FOLDER_OPT and if defined sets property
      * {@link JMeter#JMETER_REPORT_OUTPUT_DIR_PROPERTY} after checking folder can
@@ -700,7 +623,7 @@ public class JMeter implements JMeterPlugin {
             File file = new File(jsr223Init);
             if(file.exists() && file.canRead()) {
                 String extension = StringUtils.defaultIfBlank(FilenameUtils.getExtension(jsr223Init), "Groovy");
-                try (FileReader reader = new FileReader(file)) {
+                try (Reader reader = Files.newBufferedReader(file.toPath())) {
                     ScriptEngineManager scriptEngineManager = new ScriptEngineManager();
                     ScriptEngine engine = scriptEngineManager.getEngineByExtension(extension);
                     if (engine == null) {
@@ -995,7 +918,8 @@ public class JMeter implements JMeterPlugin {
     }
 
     // run test in batch mode
-     void runNonGui(String testFile, String logFile, boolean remoteStart, String remoteHostsString, boolean generateReportDashboard)
+    @SuppressWarnings("JdkObsolete")
+    void runNonGui(String testFile, String logFile, boolean remoteStart, String remoteHostsString, boolean generateReportDashboard)
             throws ConfigurationException {
         try {
             File f = new File(testFile);
@@ -1067,20 +991,20 @@ public class JMeter implements JMeterPlugin {
             // when NON GUI mode is used
             clonedTree.add(clonedTree.getArray()[0], new RemoteThreadsListenerTestElement());
 
-            List<JMeterEngine> engines = new LinkedList<>();
+            List<JMeterEngine> engines = new ArrayList<>();
             println("Created the tree successfully using "+testFile);
             if (!remoteStart) {
                 JMeterEngine engine = new StandardJMeterEngine();
                 clonedTree.add(clonedTree.getArray()[0], new ListenToTest(
                         org.apache.jmeter.JMeter.ListenToTest.RunMode.LOCAL, false, reportGenerator));
                 engine.configure(clonedTree);
-                long now=System.currentTimeMillis();
-                println("Starting standalone test @ "+new Date(now)+" ("+now+")");
+                Instant now = Instant.now();
+                println("Starting standalone test @ "+ formatLikeDate(now) + " (" + now.toEpochMilli() + ')');
                 engines.add(engine);
                 engine.runTest();
             } else {
                 java.util.StringTokenizer st = new java.util.StringTokenizer(remoteHostsString.trim(), ",");//$NON-NLS-1$
-                List<String> hosts = new LinkedList<>();
+                List<String> hosts = new ArrayList<>();
                 while (st.hasMoreElements()) {
                     hosts.add(((String) st.nextElement()).trim());
                 }
@@ -1105,6 +1029,14 @@ public class JMeter implements JMeterPlugin {
         }
     }
 
+    private static String formatLikeDate(Instant instant) {
+        return DateTimeFormatter
+                .ofLocalizedDateTime(FormatStyle.LONG)
+                .withLocale(Locale.ROOT)
+                .withZone(ZoneId.systemDefault())
+                .format(instant);
+    }
+
     /**
      * This function does the following:
      * <ul>
@@ -1153,8 +1085,7 @@ public class JMeter implements JMeterPlugin {
      * @param tree The {@link HashTree} to convert
      */
     private static void pConvertSubTree(HashTree tree) {
-        LinkedList<Object> copyList = new LinkedList<>(tree.list());
-        for (Object o  : copyList) {
+        for (Object o : new ArrayList<>(tree.list())) {
             if (o instanceof TestElement) {
                 TestElement item = (TestElement) o;
                 if (item.isEnabled()) {
@@ -1302,12 +1233,13 @@ public class JMeter implements JMeterPlugin {
             }
         }
 
+        @SuppressWarnings("JdkObsolete")
         private void endTest(boolean isDistributed) {
-            long now = System.currentTimeMillis();
+            Instant now = Instant.now();
             if (isDistributed) {
-                println("Tidying up remote @ "+new Date(now)+" ("+now+")");
+                println("Tidying up remote @ " + formatLikeDate(now) + " (" + now.toEpochMilli() + ')');
             } else {
-                println("Tidying up ...    @ "+new Date(now)+" ("+now+")");
+                println("Tidying up ...    @ " + formatLikeDate(now) + " (" + now.toEpochMilli() + ')');
             }
 
             if (isDistributed) {
@@ -1348,7 +1280,7 @@ public class JMeter implements JMeterPlugin {
             // This cannot be a JMeter class variable, because properties
             // are not initialised until later.
             final int pauseToCheckForRemainingThreads =
-                    JMeterUtils.getPropDefault("jmeter.exit.check.pause", 2000); // $NON-NLS-1$
+                    JMeterUtils.getPropDefault("jmeter.exit.check.pause", 0); // $NON-NLS-1$
 
             if (pauseToCheckForRemainingThreads > 0) {
                 Thread daemon = new Thread(){
@@ -1380,6 +1312,7 @@ public class JMeter implements JMeterPlugin {
     }
 
     @Override
+    @SuppressWarnings("JdkObsolete")
     public String[][] getIconMappings() {
         final String defaultIconProp = "org/apache/jmeter/images/icon.properties"; //$NON-NLS-1$
         final String iconSize = JMeterUtils.getPropDefault(TREE_ICON_SIZE, DEFAULT_TREE_ICON_SIZE);
